@@ -9,9 +9,8 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.3)
+# Language : Python (>= 2.5)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: constants.py 1097 2007-01-31 05:14:07Z pronovic $
 # Purpose  : Provides common constants used by standard actions.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -22,7 +21,8 @@
 
 """
 Provides common constants used by standard actions.
-@sort: DIR_TIME_FORMAT, DIGEST_EXTENSION, COLLECT_INDICATOR, STAGE_INDICATOR, STORE_INDICATOR
+@sort: DIR_TIME_FORMAT, DIGEST_EXTENSION, INDICATOR_PATTERN,
+       COLLECT_INDICATOR, STAGE_INDICATOR, STORE_INDICATOR
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
 """
 
@@ -32,8 +32,9 @@ Provides common constants used by standard actions.
 
 DIR_TIME_FORMAT      = "%Y/%m/%d"
 DIGEST_EXTENSION     = "sha"
+
+INDICATOR_PATTERN    = [ "cback\..*", ]
 COLLECT_INDICATOR    = "cback.collect"
 STAGE_INDICATOR      = "cback.stage"
 STORE_INDICATOR      = "cback.store"
 
-
