@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2005 Kenneth J. Pronovici.
+# Copyright (c) 2005,2010 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -25,13 +25,11 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.3)
+# Language : Python (>= 2.5)
 # Project  : Official Cedar Backup Extensions
-# Revision : $Id: mysql.py 663 2005-03-07 04:48:07Z pronovic $
 # Purpose  : Provides an extension to back up MySQL databases.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
-# This file was created with a width of 132 characters, and NO tabs.
 
 ########################################################################
 # Module documentation
@@ -41,7 +39,7 @@
 Provides an extension to back up MySQL databases.
 
 This is a Cedar Backup extension used to back up MySQL databases via the Cedar
-Backup command line.  It requires a new configurations section <mysql> and is
+Backup command line.  It requires a new configuration section <mysql> and is
 intended to be run either immediately before or immediately after the standard
 collect action.  Aside from its own configuration, it requires the options and
 collect configuration sections in the standard Cedar Backup configuration file.
@@ -49,23 +47,36 @@ collect configuration sections in the standard Cedar Backup configuration file.
 The backup is done via the C{mysqldump} command included with the MySQL
 product.  Output can be compressed using C{gzip} or C{bzip2}.  Administrators
 can configure the extension either to back up all databases or to back up only
-specific databases.  The extension assumes that all databases can be backed up
-by a single user (typically "root").
+specific databases.  Note that this code always produces a full backup.  There
+is currently no facility for making incremental backups.  If/when someone has a
+need for this and can describe how to do it, I'll update this extension or
+provide another.
 
-Note that this code always produces a full backup.  There is currently no
-facility for making incremental backups.  If/when someone has a need for this
-and can describe how to do it, I'll update this extension or provide another.
+The extension assumes that all configured databases can be backed up by a
+single user.  Often, the "root" database user will be used.  An alternative is
+to create a separate MySQL "backup" user and grant that user rights to read
+(but not write) various databases as needed.  This second option is probably
+the best choice.
 
-You should always make C{/etc/cback.conf} unreadble to non-root users once you
-place mysql configuration into it, since mysql configuration will contain
-information about available MySQL databases, usernames and passwords.
+The extension accepts a username and password in configuration.  However, you
+probably do not want to provide those values in Cedar Backup configuration.
+This is because Cedar Backup will provide these values to C{mysqldump} via the
+command-line C{--user} and C{--password} switches, which will be visible to
+other users in the process listing.
 
-Unfortunately, use of this extension I{will} expose usernames and passwords in
-the process listing (via C{ps}) when the backup is running.  This is because
-none of the official MySQL backup scripts provide a good way to specify
-password other than via the C{--password} command-line option.  The only
-workaround I can come up with would be to manipulate program I/O interactively
-through a pipe, which is a real pain.
+Instead, you should configure the username and password in one of MySQL's
+configuration files.  Typically, that would be done by putting a stanza like
+this in C{/root/.my.cnf}::
+
+   [mysqldump]
+   user     = root
+   password = <secret>
+
+Regardless of whether you are using C{~/.my.cnf} or C{/etc/cback.conf} to store
+database login and password information, you should be careful about who is
+allowed to view that information.  Typically, this means locking down
+permissions so that only the file owner can read the file contents (i.e. use
+mode C{0600}).
 
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
 """
@@ -80,20 +91,12 @@ import logging
 from gzip import GzipFile
 from bz2 import BZ2File
 
-# XML-related modules
-from xml.dom.ext.reader import PyExpat
-from xml.xpath import Evaluate
-from xml.parsers.expat import ExpatError
-from xml.dom.minidom import Node
-from xml.dom.minidom import getDOMImplementation
-from xml.dom.minidom import parseString
-from xml.dom.ext import PrettyPrint
-
 # Cedar Backup modules
-from CedarBackup2.config import addContainerNode, addStringNode, addBooleanNode
-from CedarBackup2.config import readChildren, readFirstChild, readString, readStringList, readBoolean
-from CedarBackup2.config import VALID_COLLECT_MODES, VALID_COMPRESS_MODES
-from CedarBackup2.util import executeCommand, ObjectTypeList, changeOwnership
+from CedarBackup2.xmlutil import createInputDom, addContainerNode, addStringNode, addBooleanNode
+from CedarBackup2.xmlutil import readFirstChild, readString, readStringList, readBoolean
+from CedarBackup2.config import VALID_COMPRESS_MODES
+from CedarBackup2.util import resolveCommand, executeCommand
+from CedarBackup2.util import ObjectTypeList, changeOwnership
 
 
 ########################################################################
@@ -117,7 +120,6 @@ class MysqlConfig(object):
 
    The following restrictions exist on data in this class:
 
-      - The user and password must always be filled in
       - The compress mode must be one of the values in L{VALID_COMPRESS_MODES}.
       - The 'all' flag must be 'Y' if no databases are defined.
       - The 'all' flag must be 'N' if any databases are defined.
@@ -126,7 +128,7 @@ class MysqlConfig(object):
    @sort: __init__, __repr__, __str__, __cmp__, user, password, all, databases
    """
 
-   def __init__(self, user=None, password=None, compressMode=None, all=None, databases=None):
+   def __init__(self, user=None, password=None, compressMode=None, all=None, databases=None):  # pylint: disable=W0622
       """
       Constructor for the C{MysqlConfig} class.
       
@@ -167,28 +169,28 @@ class MysqlConfig(object):
       """
       if other is None:
          return 1
-      if self._user != other._user:
-         if self._user < other._user:
+      if self.user != other.user:
+         if self.user < other.user:
             return -1
          else:
             return 1
-      if self._password != other._password:
-         if self._password < other._password:
+      if self.password != other.password:
+         if self.password < other.password:
             return -1
          else:
             return 1
-      if self._compressMode != other._compressMode:
-         if self._compressMode < other._compressMode:
+      if self.compressMode != other.compressMode:
+         if self.compressMode < other.compressMode:
             return -1
          else:
             return 1
-      if self._all != other._all:
-         if self._all < other._all:
+      if self.all != other.all:
+         if self.all < other.all:
             return -1
          else:
             return 1
-      if self._databases != other._databases:
-         if self._databases < other._databases:
+      if self.databases != other.databases:
+         if self.databases < other.databases:
             return -1
          else:
             return 1
@@ -380,8 +382,8 @@ class LocalConfig(object):
       """
       if other is None:
          return 1
-      if self._mysql != other._mysql:
-         if self._mysql < other._mysql:
+      if self.mysql != other.mysql:
+         if self.mysql < other.mysql:
             return -1
          else:
             return 1
@@ -412,18 +414,14 @@ class LocalConfig(object):
       """
       Validates configuration represented by the object.
 
-      The user, password and compress mode must be filled in.  Then, if the 'all'
-      flag I{is} set, no databases are allowed, and if the 'all' flag is I{not}
-      set, at least one database is required.
+      The compress mode must be filled in.  Then, if the 'all' flag I{is} set,
+      no databases are allowed, and if the 'all' flag is I{not} set, at least
+      one database is required.
 
       @raise ValueError: If one of the validations fails.
       """
       if self.mysql is None:
          raise ValueError("Mysql section is required.")
-      if self.mysql.user is None:
-         raise ValueError("Mysql user value is required.")
-      if self.mysql.password is None:
-         raise ValueError("Mysql password value is required.")
       if self.mysql.compressMode is None:
          raise ValueError("Compress mode value is required.")
       if self.mysql.all:
@@ -431,7 +429,7 @@ class LocalConfig(object):
             raise ValueError("Databases cannot be specified if 'all' flag is set.")
       else:
          if self.mysql.databases is None or len(self.mysql.databases) < 1:
-            raise ValueError("At least one database must be indicated if 'all' flag is not set.")
+            raise ValueError("At least one MySQL database must be indicated if 'all' flag is not set.")
 
    def addConfig(self, xmlDom, parentNode):
       """
@@ -477,14 +475,11 @@ class LocalConfig(object):
 
       @raise ValueError: If the XML cannot be successfully parsed.
       """
-      try:
-         xmlDom = PyExpat.Reader().fromString(xmlData)
-         parent = readFirstChild(xmlDom, "cb_config")
-         self._mysql = LocalConfig._parseMysql(parent)
-      except (IOError, ExpatError), e:
-         raise ValueError("Unable to parse XML document: %s" % e)
+      (xmlDom, parentNode) = createInputDom(xmlData)
+      self._mysql = LocalConfig._parseMysql(parentNode)
 
-   def _parseMysql(parent):
+   @staticmethod
+   def _parseMysql(parentNode):
       """
       Parses a mysql configuration section.
       
@@ -500,13 +495,13 @@ class LocalConfig(object):
 
          databases      //cb_config/mysql/database
 
-      @param parent: Parent node to search beneath.
+      @param parentNode: Parent node to search beneath.
 
       @return: C{MysqlConfig} object or C{None} if the section does not exist.
       @raise ValueError: If some filled-in value is invalid.
       """
       mysql = None
-      section = readFirstChild(parent, "mysql")
+      section = readFirstChild(parentNode, "mysql")
       if section is not None:
          mysql = MysqlConfig()
          mysql.user = readString(section, "user")
@@ -515,7 +510,6 @@ class LocalConfig(object):
          mysql.all = readBoolean(section, "all")
          mysql.databases = readStringList(section, "database")
       return mysql
-   _parseMysql = staticmethod(_parseMysql)
 
 
 ########################################################################
@@ -567,8 +561,8 @@ def _backupDatabase(targetDir, compressMode, user, password, backupUser, backupG
 
    @param targetDir:  Directory into which backups should be written.
    @param compressMode: Compress mode to be used for backed-up files.
-   @param user: User to use for connecting to the database.
-   @param password: Password associated with user.
+   @param user: User to use for connecting to the database (if any).
+   @param password: Password associated with user (if any).
    @param backupUser: User to own resulting file.
    @param backupGroup: Group to own resulting file.
    @param database: Name of database, or C{None} for all databases.
@@ -625,22 +619,44 @@ def backupDatabase(user, password, backupFile, database=None):
    Backs up an individual MySQL database, or all databases.
 
    This function backs up either a named local MySQL database or all local
-   MySQL databases, using the passed in user and password for connectivity.
-   This is I{always} a full backup.  There is no facility for incremental
-   backups.
+   MySQL databases, using the passed-in user and password (if provided) for
+   connectivity.  This function call I{always} results a full backup.  There is
+   no facility for incremental backups.
 
-   The backup data will be written into the passed-in back file.  Normally,
+   The backup data will be written into the passed-in backup file.  Normally,
    this would be an object as returned from C{open()}, but it is possible to
    use something like a C{GzipFile} to write compressed output.  The caller is
    responsible for closing the passed-in backup file.
 
-   @note: Typically, you would use the C{root} user to back up all databases.
+   Often, the "root" database user will be used when backing up all databases.
+   An alternative is to create a separate MySQL "backup" user and grant that
+   user rights to read (but not write) all of the databases that will be backed
+   up.
 
-   @param user: User to use for connecting to the database.
-   @type user: String representing MySQL username.
+   This function accepts a username and password.  However, you probably do not
+   want to pass those values in.  This is because they will be provided to
+   C{mysqldump} via the command-line C{--user} and C{--password} switches,
+   which will be visible to other users in the process listing.
 
-   @param password: Password associated with user.
-   @type password: String representing MySQL password.
+   Instead, you should configure the username and password in one of MySQL's
+   configuration files.  Typically, this would be done by putting a stanza like
+   this in C{/root/.my.cnf}, to provide C{mysqldump} with the root database
+   username and its password::
+
+      [mysqldump]
+      user     = root
+      password = <secret>
+
+   If you are executing this function as some system user other than root, then
+   the C{.my.cnf} file would be placed in the home directory of that user.  In
+   either case, make sure to set restrictive permissions (typically, mode
+   C{0600}) on C{.my.cnf} to make sure that other users cannot read the file.
+
+   @param user: User to use for connecting to the database (if any)
+   @type user: String representing MySQL username, or C{None}
+
+   @param password: Password associated with user (if any)
+   @type password: String representing MySQL password, or C{None}
 
    @param backupFile: File use for writing backup.
    @type backupFile: Python file object as from C{open()} or C{file()}.
@@ -651,15 +667,20 @@ def backupDatabase(user, password, backupFile, database=None):
    @raise ValueError: If some value is missing or invalid.
    @raise IOError: If there is a problem executing the MySQL dump.
    """
-   if user is None or password is None:
-      raise ValueError("User and password are required.")
-   args = [ "-all", "--flush-logs", "--opt", "--user=%s" % user, "--password=%s" % password, ]
+   args = [ "-all", "--flush-logs", "--opt", ]
+   if user is not None:
+      logger.warn("Warning: MySQL username will be visible in process listing (consider using ~/.my.cnf).")
+      args.append("--user=%s" % user)
+   if password is not None:
+      logger.warn("Warning: MySQL password will be visible in process listing (consider using ~/.my.cnf).")
+      args.append("--password=%s" % password)
    if database is None:
       args.insert(0, "--all-databases")
    else:
       args.insert(0, "--databases")
       args.append(database)
-   result = executeCommand(MYSQLDUMP_COMMAND, args, returnOutput=False, ignoreStderr=True, doNotLog=True, outputFile=backupFile)[0]
+   command = resolveCommand(MYSQLDUMP_COMMAND)
+   result = executeCommand(command, args, returnOutput=False, ignoreStderr=True, doNotLog=True, outputFile=backupFile)[0]
    if result != 0:
       if database is None:
          raise IOError("Error [%d] executing MySQL database dump for all databases." % result)
