@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2004-2006,2008 Kenneth J. Pronovici.
+# Copyright (c) 2004-2006,2008,2010 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -25,9 +25,8 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.3)
+# Language : Python (>= 2.5)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: testutil.py 845 2008-02-07 15:21:36Z pronovic $
 # Purpose  : Provides unit-testing utilities.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -52,8 +51,8 @@ making them available to others.
 @sort: findResources, commandAvailable,
        buildPath, removedir, extractTar, changeFileAge,
        getMaskAsMode, getLogin, failUnlessAssignRaises, runningAsRoot,
-       platformMacOsX, platformWindows, platformHasEcho, 
-       platformSupportsLinks, platformSupportsPermissions,
+       platformDebian, platformMacOsX, platformCygwin, platformWindows, 
+       platformHasEcho, platformSupportsLinks, platformSupportsPermissions,
        platformRequiresBinaryRead
 
 @author: Kenneth J. Pronovici <pronovic@ieee.org>
@@ -70,12 +69,15 @@ import tarfile
 import time
 import getpass
 import random
-import string
+import string # pylint: disable=W0402
 import platform
 import logging
 from StringIO import StringIO
 
 from CedarBackup2.util import encodePath, executeCommand
+from CedarBackup2.config import Config, OptionsConfig
+from CedarBackup2.customize import customizeOverrides
+from CedarBackup2.cli import setupPathResolver
 
 
 ########################################################################
@@ -105,6 +107,33 @@ def setupDebugLogger():
    logger.addHandler(handler)
 
 
+#################
+# setupOverrides
+#################
+
+def setupOverrides():
+   """
+   Set up any platform-specific overrides that might be required.
+
+   When packages are built, this is done manually (hardcoded) in customize.py
+   and the overrides are set up in cli.cli().  This way, no runtime checks need
+   to be done.  This is safe, because the package maintainer knows exactly
+   which platform (Debian or not) the package is being built for.  
+
+   Unit tests are different, because they might be run anywhere.  So, we
+   attempt to make a guess about plaform using platformDebian(), and use that
+   to set up the custom overrides so that platform-specific unit tests continue
+   to work.
+   """
+   config = Config()
+   config.options = OptionsConfig()
+   if platformDebian():
+      customizeOverrides(config, platform="debian")
+   else:
+      customizeOverrides(config, platform="standard")
+   setupPathResolver(config)
+
+
 ###########################
 # findResources() function
 ###########################
@@ -120,7 +149,7 @@ def findResources(resources, dataDirs):
    mapping = { }
    for resource in resources:
       for resourceDir in dataDirs:
-         path = os.path.join(resourceDir, resource);
+         path = os.path.join(resourceDir, resource)
          if os.path.exists(path):
             mapping[resource] = path
             break
@@ -204,10 +233,14 @@ def extractTar(tmpdir, filepath):
    @param filepath: Path to tarfile to extract.
    @raise ValueError: If a path cannot be encoded properly.
    """
+   # pylint: disable=E1101
    tmpdir = encodePath(tmpdir)
    filepath = encodePath(filepath)
    tar = tarfile.open(filepath)
-   tar.posix = False
+   try:
+      tar.format = tarfile.GNU_FORMAT
+   except AttributeError:
+      tar.posix = False
    for tarinfo in tar:
       tar.extract(tarinfo, tmpdir)
 
@@ -219,16 +252,25 @@ def extractTar(tmpdir, filepath):
 def changeFileAge(filename, subtract=None):
    """
    Changes a file age using the C{os.utime} function.
+
+   @note: Some platforms don't seem to be able to set an age precisely.  As a
+   result, whereas we might have intended to set an age of 86400 seconds, we
+   actually get an age of 86399.375 seconds.  When util.calculateFileAge()
+   looks at that the file, it calculates an age of 0.999992766204 days, which
+   then gets truncated down to zero whole days.  The tests get very confused.
+   To work around this, I always subtract off one additional second as a fudge
+   factor.  That way, the file age will be I{at least} as old as requested
+   later on.
+
    @param filename: File to operate on.
    @param subtract: Number of seconds to subtract from the current time.
    @raise ValueError: If a path cannot be encoded properly.
    """
    filename = encodePath(filename)
-   if subtract is None:
-      os.utime(filename, None)
-   else:
-      newTime = time.time() - subtract
-      os.utime(filename, (newTime, newTime))
+   newTime = time.time() - 1
+   if subtract is not None:
+      newTime -= subtract
+   os.utime(filename, (newTime, newTime))
 
 
 ###########################
@@ -270,7 +312,7 @@ def randomFilename(length, prefix=None, suffix=None):
    """
    characters = [None] * length
    for i in xrange(length):
-      characters[i] = random.choice(string.uppercase)
+      characters[i] = random.choice(string.ascii_uppercase)
    if prefix is None:
       prefix = ""
    if suffix is None:
@@ -282,7 +324,7 @@ def randomFilename(length, prefix=None, suffix=None):
 # failUnlessAssignRaises() function
 ####################################
 
-def failUnlessAssignRaises(testCase, exception, object, property, value):
+def failUnlessAssignRaises(testCase, exception, obj, prop, value):
    """
    Equivalent of C{failUnlessRaises}, but used for property assignments instead.
 
@@ -314,8 +356,8 @@ def failUnlessAssignRaises(testCase, exception, object, property, value):
 
    @param testCase: PyUnit test case object (i.e. self).
    @param exception: Exception that is expected to be raised.
-   @param object: Object whose property is to be assigned to.
-   @param property: Name of the property, as a string.
+   @param obj: Object whose property is to be assigned to.
+   @param prop: Name of the property, as a string.
    @param value: Value that is to be assigned to the property.
 
    @see: C{unittest.TestCase.failUnlessRaises}
@@ -323,10 +365,11 @@ def failUnlessAssignRaises(testCase, exception, object, property, value):
    missed = False
    instead = None
    try:
-      exec "object.%s = value" % property
+      exec "obj.%s = value" % prop    # pylint: disable=W0122
       missed = True
    except exception: pass
-   except Exception, e: instead = e
+   except Exception, e: 
+      instead = e
    if missed:
       testCase.fail("Expected assignment to raise %s, but got no exception." % (exception.__name__))
    if instead is not None:
@@ -337,7 +380,7 @@ def failUnlessAssignRaises(testCase, exception, object, property, value):
 # captureOutput() function
 ###########################
 
-def captureOutput(callable):
+def captureOutput(c):
    """
    Captures the output (stdout, stderr) of a function or a method.
 
@@ -351,12 +394,12 @@ def captureOutput(callable):
    @note: This method assumes that C{callable} doesn't take any arguments
    besides keyword argument C{fd} to specify the file descriptor.
 
-   @param callable: Callable function or method.
+   @param c: Callable function or method.
 
    @return: Output of function, as one big string.
    """
    fd = StringIO()
-   callable(fd=fd)
+   c(fd=fd)
    result = fd.getvalue()
    fd.close()
    return result
@@ -375,10 +418,25 @@ def _isPlatform(name):
       return platform.platform(True, True).startswith("Windows")
    elif name == "macosx":
       return sys.platform == "darwin"
+   elif name == "debian":
+      return platform.platform(False, False).find("debian") > 0
+   elif name == "cygwin":
+      return platform.platform(True, True).startswith("CYGWIN")
    else:
       raise ValueError("Unknown platform [%s]." % name)
 
 
+############################
+# platformDebian() function
+############################
+
+def platformDebian():
+   """
+   Returns boolean indicating whether this is the Debian platform.
+   """
+   return _isPlatform("debian")
+
+
 ############################
 # platformMacOsX() function
 ############################
@@ -401,6 +459,17 @@ def platformWindows():
    return _isPlatform("windows")
 
 
+############################
+# platformCygwin() function
+############################
+
+def platformCygwin():
+   """
+   Returns boolean indicating whether this is the Cygwin platform.
+   """
+   return _isPlatform("cygwin")
+
+
 ###################################
 # platformSupportsLinks() function
 ###################################
@@ -492,18 +561,18 @@ def hexFloatLiteralAllowed():
    Indicates whether hex float literals are allowed by the interpreter.
 
    As far back as 2004, some Python documentation indicated that octal and hex
-   notation applies only to integer literals.  However, prior to Python 2.5, it
-   was legal to construct a float with an argument like 0xAC.  This check
-   provides a version-based indication of whether the current interpreter
+   notation applied only to integer literals.  However, prior to Python 2.5, it
+   was legal to construct a float with an argument like 0xAC on some platforms.
+   This check provides a an indication of whether the current interpreter
    supports that behavior.
 
    This check exists so that unit tests can continue to test the same thing as
    always for pre-2.5 interpreters (i.e. making sure backwards compatibility
    doesn't break) while still continuing to work for later interpreters.
 
-   The returned value is True for Python <= 2.5, and False otherwise.
+   The returned value is True if hex float literals are allowed, False otherwise.
    """
-   if map(int, [sys.version_info[0], sys.version_info[1]]) < [2, 5]:
+   if map(int, [sys.version_info[0], sys.version_info[1]]) < [2, 5] and not platformWindows():
       return True
    return False
 
