@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2007 Kenneth J. Pronovici.
+# Copyright (c) 2007-2008,2010 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -25,9 +25,8 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.3)
+# Language : Python (>= 2.5)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: span.py 1136 2007-02-23 05:25:10Z pronovic $
 # Purpose  : Spans staged data among multiple discs
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -66,13 +65,14 @@ from CedarBackup2.release import AUTHOR, EMAIL, VERSION, DATE, COPYRIGHT
 from CedarBackup2.util import displayBytes, convertSize, mount, unmount
 from CedarBackup2.util import UNIT_SECTORS, UNIT_BYTES
 from CedarBackup2.config import Config
-from CedarBackup2.filesystem import FilesystemList, BackupFileList, compareDigestMaps, normalizeDir
+from CedarBackup2.filesystem import BackupFileList, compareDigestMaps, normalizeDir
 from CedarBackup2.cli import Options, setupLogging, setupPathResolver
 from CedarBackup2.cli import DEFAULT_CONFIG, DEFAULT_LOGFILE, DEFAULT_OWNERSHIP, DEFAULT_MODE
 from CedarBackup2.actions.constants import STORE_INDICATOR
-from CedarBackup2.actions.store import createWriter, consistencyCheck, writeStoreIndicator
+from CedarBackup2.actions.util import createWriter
+from CedarBackup2.actions.store import writeIndicatorFile
 from CedarBackup2.actions.util import findDailyDirs
-from CedarBackup2.knapsack import firstFit, bestFit, worstFit, alternateFit
+from CedarBackup2.util import Diagnostics
 
 
 ########################################################################
@@ -130,7 +130,7 @@ def cli():
 
    A different error code is returned for each type of failure:
 
-      - C{1}: The Python interpreter version is < 2.3
+      - C{1}: The Python interpreter version is < 2.5
       - C{2}: Error processing command-line arguments
       - C{3}: Error configuring logging
       - C{4}: Error parsing indicated configuration file
@@ -144,12 +144,12 @@ def cli():
    @return: Error code as described above.
    """
    try:
-      if map(int, [sys.version_info[0], sys.version_info[1]]) < [2, 3]:
-         sys.stderr.write("Python version 2.3 or greater required.\n")
+      if map(int, [sys.version_info[0], sys.version_info[1]]) < [2, 5]:
+         sys.stderr.write("Python version 2.5 or greater required.\n")
          return 1
    except:
       # sys.version_info isn't available before 2.0
-      sys.stderr.write("Python version 2.3 or greater required.\n")
+      sys.stderr.write("Python version 2.5 or greater required.\n")
       return 1
 
    try:
@@ -165,6 +165,9 @@ def cli():
    if options.version:
       _version()
       return 0
+   if options.diagnostics:
+      _diagnostics()
+      return 0
 
    try:
       logfile = setupLogging(options)
@@ -271,6 +274,23 @@ def _version(fd=sys.stdout):
    fd.write("\n")
 
 
+##########################
+# _diagnostics() function
+##########################
+
+def _diagnostics(fd=sys.stdout):
+   """
+   Prints runtime diagnostics information.
+   @param fd: File descriptor used to print information.
+   @note: The C{fd} is used rather than C{print} to facilitate unit testing.
+   """
+   fd.write("\n")
+   fd.write("Diagnostics:\n")
+   fd.write("\n")
+   Diagnostics().printDiagnostics(fd=fd, prefix="   ")
+   fd.write("\n")
+
+
 ############################
 # _executeAction() function
 ############################
@@ -288,9 +308,9 @@ def _executeAction(options, config):
    @raise Exception: Under many generic error conditions
    """
    print ""
-   print "================================================";
+   print "================================================"
    print "           Cedar Backup 'span' tool"
-   print "================================================";
+   print "================================================"
    print ""
    print "This the Cedar Backup span tool.  It is used to split up staging"
    print "data when that staging data does not fit onto a single disc."
@@ -353,7 +373,7 @@ def _executeAction(options, config):
    print "==="
 
    realCapacity = ((100.0 - cushion)/100.0) * mediaCapacity
-   minimumDiscs = (totalSize/realCapacity) + 1;
+   minimumDiscs = (totalSize/realCapacity) + 1
    print ""
    print "The real capacity, taking into account the %.2f%% cushion, is %s." % (cushion, displayBytes(realCapacity))
    print "It will take at least %d disc(s) to store your %s of data." % (minimumDiscs, displayBytes(totalSize))
@@ -378,7 +398,7 @@ def _executeAction(options, config):
       print "If you don't like the results you will have a chance to try a"
       print "different one later."
       print ""
-      algorithm = _getChoiceAnswer("Which algorithm?", "worst", [ "first", "best", "worst", "alternate",])
+      algorithm = _getChoiceAnswer("Which algorithm?", "worst", [ "first", "best", "worst", "alternate", ])
       print "==="
 
       print ""
@@ -400,19 +420,20 @@ def _executeAction(options, config):
          happy = True
       print "==="
 
-   print ""
-   _getReturn("Please place the first disc in your backup device.\nPress return when ready.")
-   print "==="
-
    counter = 0
    for spanItem in spanSet:
       counter += 1
+      if counter == 1:
+         print ""
+         _getReturn("Please place the first disc in your backup device.\nPress return when ready.")
+         print "==="
+      else:
+         print ""
+         _getReturn("Please replace the disc in your backup device.\nPress return when ready.")
+         print "==="
       _writeDisc(config, writer, spanItem)
-      print ""
-      _getReturn("Please replace the disc in your backup device.\nPress return when ready.")
-      print "==="
 
-   writeStoreIndicator(config, dailyDirs)
+   _writeStoreIndicator(config, dailyDirs)
 
    print ""
    print "Completed writing all discs."
@@ -447,6 +468,23 @@ def _findDailyDirs(stagingDir):
    return (results, fileList)
 
 
+##################################
+# _writeStoreIndicator() function
+##################################
+
+def _writeStoreIndicator(config, dailyDirs):
+   """
+   Writes a store indicator file into daily directories.
+
+   @param config: Config object.
+   @param dailyDirs: List of daily directories
+   """
+   for dailyDir in dailyDirs:
+      writeIndicatorFile(dailyDir, STORE_INDICATOR,
+                         config.options.backupUser,
+                         config.options.backupGroup)
+
+
 ########################
 # _getWriter() function
 ########################
@@ -475,17 +513,90 @@ def _writeDisc(config, writer, spanItem):
    @param spanItem: Span item to write
    """
    print ""
-   print "Initializing image..."
-   writer.initializeImage(newDisc=True, tmpdir=config.options.workingDir)
-   for path in spanItem.fileList:
-      graftPoint = os.path.dirname(path.replace(config.store.sourceDir, "", 1))
-      writer.addImageEntry(path, graftPoint)
-   print "Writing image to disc..."
-   writer.writeImage()
-   if config.store.checkData:
-      print "Running consistency check..."
-      _consistencyCheck(config, spanItem.fileList)
+   _discInitializeImage(config, writer, spanItem)
+   _discWriteImage(config, writer)
+   _discConsistencyCheck(config, writer, spanItem)
    print "Write process is complete."
+   print "==="
+
+def _discInitializeImage(config, writer, spanItem):
+   """
+   Initialize an ISO image for a span item.
+   @param config: Cedar Backup configuration
+   @param writer: Writer to use
+   @param spanItem: Span item to write
+   """
+   complete = False
+   while not complete:
+      try:
+         print "Initializing image..."
+         writer.initializeImage(newDisc=True, tmpdir=config.options.workingDir)
+         for path in spanItem.fileList:
+            graftPoint = os.path.dirname(path.replace(config.store.sourceDir, "", 1))
+            writer.addImageEntry(path, graftPoint)
+         complete = True
+      except KeyboardInterrupt, e:
+         raise e
+      except Exception, e:
+         logger.error("Failed to initialize image: %s" % e)
+         if not _getYesNoAnswer("Retry initialization step?", default="Y"):
+            raise e
+         print "Ok, attempting retry."
+         print "==="
+   print "Completed initializing image."
+
+def _discWriteImage(config, writer):
+   """
+   Writes a ISO image for a span item.
+   @param config: Cedar Backup configuration
+   @param writer: Writer to use
+   """
+   complete = False
+   while not complete:
+      try:
+         print "Writing image to disc..."
+         writer.writeImage()
+         complete = True
+      except KeyboardInterrupt, e:
+         raise e
+      except Exception, e:
+         logger.error("Failed to write image: %s" % e)
+         if not _getYesNoAnswer("Retry this step?", default="Y"):
+            raise e
+         print "Ok, attempting retry."
+         _getReturn("Please replace media if needed.\nPress return when ready.")
+         print "==="
+   print "Completed writing image."
+
+def _discConsistencyCheck(config, writer, spanItem):
+   """
+   Run a consistency check on an ISO image for a span item.
+   @param config: Cedar Backup configuration
+   @param writer: Writer to use
+   @param spanItem: Span item to write
+   """
+   if config.store.checkData:
+      complete = False
+      while not complete:
+         try:
+            print "Running consistency check..."
+            _consistencyCheck(config, spanItem.fileList)
+            complete = True
+         except KeyboardInterrupt, e:
+            raise e
+         except Exception, e:
+            logger.error("Consistency check failed: %s" % e)
+            if not _getYesNoAnswer("Retry the consistency check?", default="Y"):
+               raise e
+            if _getYesNoAnswer("Rewrite the disc first?", default="N"):
+               print "Ok, attempting retry."
+               _getReturn("Please replace the disc in your backup device.\nPress return when ready.")
+               print "==="
+               _discWriteImage(config, writer)
+            else:
+               print "Ok, attempting retry."
+               print "==="
+      print "Completed consistency check."
 
 
 ###############################
@@ -607,6 +718,5 @@ def _getReturn(prompt):
 ########################################################################
 
 if __name__ == "__main__":
-   result = cli()
-   sys.exit(result)
+   sys.exit(cli())
 
