@@ -8,7 +8,7 @@
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
-# Copyright (c) 2004-2007 Kenneth J. Pronovici.
+# Copyright (c) 2004-2008,2010 Kenneth J. Pronovici.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
@@ -25,9 +25,8 @@
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 #
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
-# Language : Python (>= 2.3)
+# Language : Python (>= 2.5)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: cdwriter.py 747 2007-03-25 19:42:43Z pronovic $
 # Purpose  : Provides functionality related to CD writer devices.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -59,9 +58,9 @@ import os
 import re
 import logging
 import tempfile
+import time
 
 # Cedar Backup modules
-from CedarBackup2.filesystem import FilesystemList
 from CedarBackup2.util import resolveCommand, executeCommand
 from CedarBackup2.util import convertSize, displayBytes, encodePath
 from CedarBackup2.util import UNIT_SECTORS, UNIT_BYTES, UNIT_KBYTES, UNIT_MBYTES
@@ -198,7 +197,7 @@ class MediaCapacity(object):
    supported or if the disc has no boundaries) or in exactly the form provided
    by C{cdrecord -msinfo}.  It can be passed as-is to the C{IsoImage} class.
 
-   @sort: __init__, bytesUsed, bytesAvailable, boundaries
+   @sort: __init__, bytesUsed, bytesAvailable, boundaries, totalCapacity, utilized
    """
 
    def __init__(self, bytesUsed, bytesAvailable, boundaries):
@@ -215,27 +214,51 @@ class MediaCapacity(object):
       else:
          self._boundaries = (int(boundaries[0]), int(boundaries[1]))
 
+   def __str__(self):
+      """
+      Informal string representation for class instance.
+      """
+      return "utilized %s of %s (%.2f%%)" % (displayBytes(self.bytesUsed), displayBytes(self.totalCapacity), self.utilized)
+
    def _getBytesUsed(self):
       """
-      Property target used to get the bytes-used value.
+      Property target to get the bytes-used value.
       """
       return self._bytesUsed
 
    def _getBytesAvailable(self):
       """
-      Property target available to get the bytes-available value.
+      Property target to get the bytes-available value.
       """
       return self._bytesAvailable
 
    def _getBoundaries(self):
       """
-      Property target available to get the boundaries tuple.
+      Property target to get the boundaries tuple.
       """
       return self._boundaries
 
+   def _getTotalCapacity(self):
+      """
+      Property target to get the total capacity (used + available).
+      """
+      return self.bytesUsed + self.bytesAvailable
+
+   def _getUtilized(self):
+      """
+      Property target to get the percent of capacity which is utilized.
+      """
+      if self.bytesAvailable <= 0.0:
+         return 100.0
+      elif self.bytesUsed <= 0.0:
+         return 0.0
+      return (self.bytesUsed / self.totalCapacity) * 100.0
+
    bytesUsed = property(_getBytesUsed, None, None, doc="Space used on disc, in bytes.")
    bytesAvailable = property(_getBytesAvailable, None, None, doc="Space available on disc, in bytes.")
    boundaries = property(_getBoundaries, None, None, doc="Session disc boundaries, in terms of ISO sectors.")
+   totalCapacity = property(_getTotalCapacity, None, None, doc="Total capacity of the disc, in bytes.")
+   utilized = property(_getUtilized, None, None, "Percentage of the total capacity which is utilized.")
 
 
 ########################################################################
@@ -389,7 +412,8 @@ class CdWriter(object):
    ##############
 
    def __init__(self, device, scsiId=None, driveSpeed=None, 
-                mediaType=MEDIA_CDRW_74, noEject=False, unittest=False):
+                mediaType=MEDIA_CDRW_74, noEject=False, 
+                refreshMediaDelay=0, ejectDelay=0, unittest=False):
       """
       Initializes a CD writer object.
 
@@ -430,6 +454,12 @@ class CdWriter(object):
       @param noEject: Overrides properties to indicate that the device does not support eject.
       @type noEject: Boolean true/false
 
+      @param refreshMediaDelay: Refresh media delay to use, if any
+      @type refreshMediaDelay: Number of seconds, an integer >= 0
+
+      @param ejectDelay: Eject delay to use, if any
+      @type ejectDelay: Number of seconds, an integer >= 0
+
       @param unittest: Turns off certain validations, for use in unit testing.
       @type unittest: Boolean true/false
 
@@ -444,6 +474,8 @@ class CdWriter(object):
       self._driveSpeed = validateDriveSpeed(driveSpeed)
       self._media = MediaDefinition(mediaType)
       self._noEject = noEject
+      self._refreshMediaDelay = refreshMediaDelay
+      self._ejectDelay = ejectDelay
       if not unittest:
          (self._deviceType,
           self._deviceVendor,
@@ -532,9 +564,21 @@ class CdWriter(object):
       """
       return self._deviceCanEject
 
+   def _getRefreshMediaDelay(self):
+      """
+      Property target used to get the configured refresh media delay, in seconds.
+      """
+      return self._refreshMediaDelay
+
+   def _getEjectDelay(self):
+      """
+      Property target used to get the configured eject delay, in seconds.
+      """
+      return self._ejectDelay
+
    device = property(_getDevice, None, None, doc="Filesystem device name for this writer.")
    scsiId = property(_getScsiId, None, None, doc="SCSI id for the device, in the form C{[<method>:]scsibus,target,lun}.")
-   hardwareId = property(_getHardwareId, None, None, doc="Hardware id for this writer, either SCSI id or device path.");
+   hardwareId = property(_getHardwareId, None, None, doc="Hardware id for this writer, either SCSI id or device path.")
    driveSpeed = property(_getDriveSpeed, None, None, doc="Speed at which the drive writes.")
    media = property(_getMedia, None, None, doc="Definition of media that is expected to be in the device.")
    deviceType = property(_getDeviceType, None, None, doc="Type of the device, as returned from C{cdrecord -prcap}.")
@@ -544,6 +588,8 @@ class CdWriter(object):
    deviceSupportsMulti = property(_getDeviceSupportsMulti, None, None, doc="Indicates whether device supports multisession discs.")
    deviceHasTray = property(_getDeviceHasTray, None, None, doc="Indicates whether the device has a media tray.")
    deviceCanEject = property(_getDeviceCanEject, None, None, doc="Indicates whether the device supports ejecting its media.")
+   refreshMediaDelay = property(_getRefreshMediaDelay, None, None, doc="Refresh media delay, in seconds.")
+   ejectDelay = property(_getEjectDelay, None, None, doc="Eject delay, in seconds.")
 
 
    #################################################
@@ -649,6 +695,7 @@ class CdWriter(object):
             logger.debug("Returning disc boundaries: (%d, %d)" % (boundaries[0], boundaries[1]))
          return boundaries
 
+   @staticmethod
    def _calculateCapacity(media, boundaries):
       """
       Calculates capacity for the media in terms of boundaries.
@@ -677,7 +724,6 @@ class CdWriter(object):
          bytesAvailable = convertSize(sectorsAvailable, UNIT_SECTORS, UNIT_BYTES)
       logger.debug("Used [%s], available [%s]." % (displayBytes(bytesUsed), displayBytes(bytesAvailable)))
       return MediaCapacity(bytesUsed, bytesAvailable, boundaries)
-   _calculateCapacity = staticmethod(_calculateCapacity)
 
 
    #######################################################
@@ -772,15 +818,47 @@ class CdWriter(object):
 
       If the writer was constructed with C{noEject=True}, then this is a no-op.
 
+      Starting with Debian wheezy on my backup hardware, I started seeing
+      consistent problems with the eject command.  I couldn't tell whether
+      these problems were due to the device management system or to the new
+      kernel (3.2.0).  Initially, I saw simple eject failures, possibly because
+      I was opening and closing the tray too quickly.  I worked around that
+      behavior with the new ejectDelay flag.  
+      
+      Later, I sometimes ran into issues after writing an image to a disc:
+      eject would give errors like "unable to eject, last error: Inappropriate
+      ioctl for device".  Various sources online (like Ubuntu bug #875543)
+      suggested that the drive was being locked somehow, and that the
+      workaround was to run 'eject -i off' to unlock it.  Sure enough, that
+      fixed the problem for me, so now it's a normal error-handling strategy.
+
       @raise IOError: If there is an error talking to the device.
       """
       if not self._noEject:
          if self._deviceHasTray and self._deviceCanEject:
             args = CdWriter._buildOpenTrayArgs(self._device)
-            command = resolveCommand(EJECT_COMMAND)
-            result = executeCommand(command, args)[0]
+            result = executeCommand(EJECT_COMMAND, args)[0]
             if result != 0:
-               raise IOError("Error (%d) executing eject command to open tray." % result)
+               logger.debug("Eject failed; attempting kludge of unlocking the tray before retrying.")
+               self.unlockTray()
+               result = executeCommand(EJECT_COMMAND, args)[0]
+               if result != 0:
+                  raise IOError("Error (%d) executing eject command to open tray (failed even after unlocking tray)." % result)
+               logger.debug("Kludge was apparently successful.")
+            if self.ejectDelay is not None:
+               logger.debug("Per configuration, sleeping %d seconds after opening tray." % self.ejectDelay)
+               time.sleep(self.ejectDelay)
+
+   def unlockTray(self):
+      """
+      Unlocks the device's tray.
+      @raise IOError: If there is an error talking to the device.
+      """
+      args = CdWriter._buildUnlockTrayArgs(self._device)
+      command = resolveCommand(EJECT_COMMAND)
+      result = executeCommand(command, args)[0]
+      if result != 0:
+         raise IOError("Error (%d) executing eject command to unlock tray." % result)
 
    def closeTray(self):
       """
@@ -811,18 +889,25 @@ class CdWriter(object):
 
       Sometimes, a device gets confused about the state of its media.  Often,
       all it takes to solve the problem is to eject the media and then
-      immediately reload it.  
+      immediately reload it.  (There are also configurable eject and refresh
+      media delays which can be applied, for situations where this makes a
+      difference.)
 
       This only works if the device has a tray and supports ejecting its media.
       We have no way to know if the tray is currently open or closed, so we
       just send the appropriate command and hope for the best.  If the device
       does not have a tray or does not support ejecting its media, then we do
-      nothing.
+      nothing.  The configured delays still apply, though.
 
       @raise IOError: If there is an error talking to the device.
       """
       self.openTray()
       self.closeTray()
+      self.unlockTray()  # on some systems, writing a disc leaves the tray locked, yikes!
+      if self.refreshMediaDelay is not None:
+         logger.debug("Per configuration, sleeping %d seconds to stabilize media state." % self.refreshMediaDelay)
+         time.sleep(self.refreshMediaDelay)
+      logger.debug("Media refresh complete; hopefully media state is stable now.")
 
    def writeImage(self, imagePath=None, newDisc=False, writeMulti=True):
       """
@@ -887,8 +972,8 @@ class CdWriter(object):
       capacity = self.retrieveCapacity(entireDisc=self._image.newDisc)
       image = IsoImage(self.device, capacity.boundaries)
       image.volumeId = self._image.mediaLabel  # may be None, which is also valid
-      for path in self._image.entries.keys():
-         image.addEntry(path, self._image.entries[path], override=False, contentsOnly=True)
+      for key in self._image.entries.keys():
+         image.addEntry(key, self._image.entries[key], override=False, contentsOnly=True)
       size = image.getEstimatedSize()
       logger.info("Image size will be %s." % displayBytes(size))
       available = capacity.bytesAvailable
@@ -944,6 +1029,7 @@ class CdWriter(object):
    # Methods used to parse command output
    #######################################
 
+   @staticmethod
    def _parsePropertiesOutput(output):
       """
       Parses the output from a C{cdrecord} properties command.
@@ -1017,8 +1103,8 @@ class CdWriter(object):
             deviceCanEject = True
             logger.info("Device can eject its media.")
       return (deviceType, deviceVendor, deviceId, deviceBufferSize, deviceSupportsMulti, deviceHasTray, deviceCanEject)
-   _parsePropertiesOutput = staticmethod(_parsePropertiesOutput)
 
+   @staticmethod
    def _parseBoundariesOutput(output):
       """
       Parses the output from a C{cdrecord} capacity command.
@@ -1060,13 +1146,13 @@ class CdWriter(object):
       except TypeError:
          raise IOError("Unable to parse output of boundaries command.")
       return boundaries
-   _parseBoundariesOutput = staticmethod(_parseBoundariesOutput)
 
 
    #################################
    # Methods used to build commands
    #################################
 
+   @staticmethod
    def _buildOpenTrayArgs(device):
       """
       Builds a list of arguments to be passed to a C{eject} command.
@@ -1082,8 +1168,25 @@ class CdWriter(object):
       args = []
       args.append(device)
       return args
-   _buildOpenTrayArgs = staticmethod(_buildOpenTrayArgs)
 
+   @staticmethod
+   def _buildUnlockTrayArgs(device):
+      """
+      Builds a list of arguments to be passed to a C{eject} command.
+
+      The arguments will cause the C{eject} command to unlock the tray.
+
+      @param device: Filesystem device name for this writer, i.e. C{/dev/cdrw}.
+
+      @return: List suitable for passing to L{util.executeCommand} as C{args}.
+      """
+      args = []
+      args.append("-i")
+      args.append("off")
+      args.append(device)
+      return args
+
+   @staticmethod
    def _buildCloseTrayArgs(device):
       """
       Builds a list of arguments to be passed to a C{eject} command.
@@ -1100,8 +1203,8 @@ class CdWriter(object):
       args.append("-t")
       args.append(device)
       return args
-   _buildCloseTrayArgs = staticmethod(_buildCloseTrayArgs)
 
+   @staticmethod
    def _buildPropertiesArgs(hardwareId):
       """
       Builds a list of arguments to be passed to a C{cdrecord} command.
@@ -1117,8 +1220,8 @@ class CdWriter(object):
       args.append("-prcap")
       args.append("dev=%s" % hardwareId)
       return args
-   _buildPropertiesArgs = staticmethod(_buildPropertiesArgs)
 
+   @staticmethod
    def _buildBoundariesArgs(hardwareId):
       """
       Builds a list of arguments to be passed to a C{cdrecord} command.
@@ -1135,8 +1238,8 @@ class CdWriter(object):
       args.append("-msinfo")
       args.append("dev=%s" % hardwareId)
       return args
-   _buildBoundariesArgs = staticmethod(_buildBoundariesArgs)
 
+   @staticmethod
    def _buildBlankArgs(hardwareId, driveSpeed=None):
       """
       Builds a list of arguments to be passed to a C{cdrecord} command.
@@ -1158,8 +1261,8 @@ class CdWriter(object):
          args.append("speed=%d" % driveSpeed)
       args.append("dev=%s" % hardwareId)
       return args
-   _buildBlankArgs = staticmethod(_buildBlankArgs)
 
+   @staticmethod
    def _buildWriteArgs(hardwareId, imagePath, driveSpeed=None, writeMulti=True):
       """
       Builds a list of arguments to be passed to a C{cdrecord} command.
@@ -1188,5 +1291,4 @@ class CdWriter(object):
       args.append("-data")
       args.append(imagePath) 
       return args
-   _buildWriteArgs = staticmethod(_buildWriteArgs)
 
